/*
 ** private.h
 ** fixbuf IPFIX Implementation Private Interface
 **
 ** ------------------------------------------------------------------------
 ** Copyright (C) 2006-2008 Carnegie Mellon University. All Rights Reserved.
 ** ------------------------------------------------------------------------
 ** Authors: Brian Trammell <bht@cert.org>
 ** ------------------------------------------------------------------------
 ** GNU Lesser GPL Rights pursuant to Version 2.1, February 1999 
 ** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
 ** ------------------------------------------------------------------------
 */

#ifndef _FB_PRIVATE_H_
#define _FB_PRIVATE_H_
#include <fixbuf/public.h>

/**
 * @file
 *
 * fixbuf IPFIX protocol library private interface. These calls and structures
 * are intended for the use of libfixbuf modules, and as such are not 
 * documented or guaranteed to remain stable in any way. Applications using 
 * these calls and structures may have to be modified to track changes to this
 * interface across minor version releases of fixbuf.
 */

/** 
 * An IPFIX template or options template structure. Part of the private 
 * interface. Applications should use the fbTemplate calls defined in public.h 
 * to manipulate templates instead of accessing this structure directly.
 */
struct fbTemplate_st {
    /** Information model (for looking up information elements by spec) */
    fbInfoModel_t       *model;
    /** Reference count */
    int                 ref_count;
    /** Count of information elements in template. */
    uint16_t            ie_count;
    /** 
     * Count of scope information elements in template. If sie_count 
     * is greater than 0, this template is an options template.
     */
    uint16_t            scope_count;
    /**
     * Total length of information elements in records described by 
     * this template. If the is_varlen flag is set, this represents the
     * minimum length of the information elements in the record 
     * (i.e. with each variable length IE's length set to 0).     
     */
    uint16_t            ie_len;
    /**
     * Total length of the template record or options template record 
     * defining this template. Used during template input and output.
     */
    uint16_t            tmpl_len;
    /** Set to TRUE if this template contains any variable length IEs. */
    gboolean            is_varlen;
    /** Ordered array of pointers to information elements in this template. */
    fbInfoElement_t     **ie_ary;
    /** Map of information element to index in ie_ary. */
    GHashTable          *indices;
	/** Field offset cache. For internal use by the transcoder. */
	uint16_t			*off_cache;
    /** TRUE if this template has been activated (is no longer mutable) */
    gboolean            active;
};

void                fBufRewind(
    fBuf_t              *fbuf);

gboolean            fBufAppendTemplate(
    fBuf_t              *fbuf,
    uint16_t            tmpl_id,
    fbTemplate_t        *tmpl,
    gboolean            revoke,
    GError              **err);
   
uint32_t            fbInfoElementHash(
    fbInfoElement_t     *ie);

gboolean            fbInfoElementEqual(
    const fbInfoElement_t   *a,
    const fbInfoElement_t   *b);

void                fbInfoElementDebug(
    gboolean            tmpl,
    fbInfoElement_t     *ie);

const fbInfoElement_t     *fbInfoModelGetElement(
    fbInfoModel_t       *model,
    fbInfoElement_t     *ex_ie);

gboolean            fbInfoElementCopyToTemplate(
    fbInfoModel_t       *model,
    fbInfoElement_t     *ex_ie,
    fbInfoElement_t     *tmpl_ie);

gboolean            fbInfoElementCopyToTemplateByName(
    fbInfoModel_t       *model,
    const char          *name,
    uint16_t            len_override,
    fbInfoElement_t     *tmpl_ie);

void                fbTemplateRetain(
    fbTemplate_t        *tmpl);

void                fbTemplateRelease(
    fbTemplate_t        *tmpl);

void                fbTemplateFree(
    fbTemplate_t        *tmpl);

void                fbTemplateDebug(
    const char          *label,
    uint16_t            tid,
    fbTemplate_t        *tmpl);

fbSession_t         *fbSessionClone(
    fbSession_t         *base);

uint32_t            fbSessionGetSequence(
    fbSession_t         *session);

void                fbSessionSetSequence(
    fbSession_t         *session,
    uint32_t            sequence);

void                fbSessionSetTemplateBuffer(
    fbSession_t         *session,
    fBuf_t              *fbuf);

fbInfoModel_t       *fbSessionGetInfoModel(
    fbSession_t         *session);

gboolean            fbConnSpecLookupAI(
    fbConnSpec_t        *spec,
    gboolean            passive,
    GError              **err);

gboolean            fbConnSpecInitTLS(
    fbConnSpec_t        *spec,
    gboolean            passive,
    GError              **err);

fbConnSpec_t        *fbConnSpecCopy(
    fbConnSpec_t        *spec);

void                fbConnSpecFree(
    fbConnSpec_t        *spec);

uint16_t            fbExporterGetMTU(
    fbExporter_t        *exporter);
       
gboolean            fbExportMessage(
    fbExporter_t        *exporter,
    uint8_t             *msgbase,
    size_t              msglen,
    GError              **err);
    
void                fbExporterFree(
    fbExporter_t       *exporter);

fbCollector_t       *fbCollectorAllocSocket(
    fbListener_t        *listener,
    void                *ctx,
    int                 fd,
    struct sockaddr     *peer,
    size_t              peerlen);

fbCollector_t       *fbCollectorAllocTLS(
    fbListener_t        *listener,
    void                *ctx,
    int                 fd,
    struct sockaddr     *peer,
    size_t              peerlen,
    GError              **err);

gboolean            fbCollectMessage(
    fbCollector_t       *collector,
    uint8_t             *msgbase,
    size_t              *msglen,
    GError              **err);

int                 fbCollectorGetFD(
    fbCollector_t       *collector);

void                fbCollectorFree(
    fbCollector_t       *collector);

void fbListenerRemove(
    fbListener_t        *listener,
    int                 fd);

fbConnSpec_t        *fbListenerGetConnSpec(
    fbListener_t        *listener);

#endif
