/*
 ** public.h
 ** fixbuf IPFIX Implementation Public Interface
 **
 ** ------------------------------------------------------------------------
 ** Copyright (C) 2006-2008 Carnegie Mellon University. All Rights Reserved.
 ** ------------------------------------------------------------------------
 ** Authors: Brian Trammell <bht@cert.org>
 ** ------------------------------------------------------------------------
 ** GNU Lesser GPL Rights pursuant to Version 2.1, February 1999 
 ** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
 ** ------------------------------------------------------------------------
 */

/**
 * @mainpage libfixbuf - IPFIX Protocol Library
 *
 * @section Introduction
 *
 * libfixbuf aims to be a compliant implementation of the IPFIX Protocol, 
 * as defined in the "Specification of the IPFIX Protocol for the Export of IP 
 * Flow Information" (draft-ietf-ipfix-protocol, current revision -24, 
 * presently in the IETF RFC Editor queue). It supports the information model
 * defined in "Information Model for IP Flow Information Export"
 * (draft-ietf-ipfix-info, current revision -15, presently in the IETF RFC
 * Editor Queue), extended as proposed by "Bidirectional Flow Export using 
 * IPFIX" (draft-ietf-ipfix-biflow, current revision -05, presently in the IETF
 * RFC Editor Queue) to support information elements for representing biflows. 
 *
 * libfixbuf supports UDP, TCP, SCTP, and TLS over TCP as transport protocols.
 * Support for DTLS over UDP and DTLS over SCTP is forthcoming. It also 
 * supports operation as an IPFIX File Writer or IPFIX File Reader as defined 
 * in "An IPFIX-Based File Format" (draft-trammell-ipfix-file, current revision
 * -04).
 *
 * libfixbuf's public API is defined in public.h; see the documentation of 
 * that file for general documentation on getting started with libfixbuf, as 
 * well as detailed documentation on the public API calls and data types.
 *
 * @section Downloading
 *
 * libfixbuf is distributed from http://tools.netsa.cert.org/fixbuf
 *
 * <a href="http://tools.netsa.cert.org/releases/libfixbuf-0.7.2.tar.gz">Download libfixbuf 0.7.2</a>
 *
 * @section Building
 *
 * libfixbuf uses a reasonably standard autotools-based build system. 
 * The customary build procedure (<tt>./configure && make 
 * && make install</tt>) should work in most environments. 
 * 
 * libfixbuf requires glib-2.0 version 2.6.4 or later. If built against
 * version 2.10 or later, it will automatically use the glib slab allocator
 * for increased memory allocation performance. glib is available on most
 * modern Linux distributions and BSD ports collections, or in source form from
 * <a href="http://www.gtk.org">http://www.gtk.org</a>.
 *
 * libfixbuf automatically uses the getaddrinfo(3) facility and the
 * accompanying dual IPv4/IPv6 stack support if present. getaddrinfo(3)
 * must be present to export or collect flows over IPv6.
 *
 * libfixbuf does not build with SCTP support by default. The --with-sctp 
 * option must be given to the libfixbuf ./configure script to include SCTP
 * support. Also note that SCTP requires kernel support, and applications
 * built against libfixbuf with libsctp may fail at runtime if that kernel
 * support is not present.
 * 
 * libfixbuf does not build with TLS support by default. The --with-ssl option
 * must be given to the libfixbuf ./configure script to include TLS support.
 * 
 * @section Known Issues
 * 
 * Note that while libfixbuf has been tested for interoperability against other 
 * IPFIX implementations, it has not been heavily tested outside the scope of 
 * the application suite it was written for. In addition, the API may change 
 * from release to release until version 1.0.0.
 *
 * The following are known issues with libfixbuf as of version 0.7.2:
 *
 * <ul>
 * <li>There is no support for DTLS over UDP or DTLS over SCTP transport.</li>
 * <li>There is no support for application-selectable SCTP stream assignment
 *     or SCTP partial reliability. Templates are sent reliably on stream 0,
 *     and data sets are sent reliably on stream 1.
 * </li>
 * <li>UDP listener support is limited to a single remote Exporting Process;
 *     the first remote endpoint to connect to a UDP listener is the only
 *     one from which IPFIX Messages will be accepted for the listener's
 *     lifetime. This is an obvious security flaw and limits the usefulness
 *     of libfixbuf for implementing UDP collecting processes. A peer address
 *     based session demultiplexer is planned for a future release, but may
 *     require incompatible API changes.
 * </li>
 * <li>There is no automatic support for periodic template retransmission
 *     or periodic template expiration as required when transporting IPFIX
 *     over UDP. Applications using libfixbuf to transport IPFIX messages 
 *     over UDP must maintain these timeouts and manually manage the session.
 * </li>
 * </ul>
 *
 * @section Copyright
 *
 * libfixbuf is copyright 2005-2008 Carnegie Mellon University, and is released
 * under the GNU Lesser General Public License. See the COPYING file in the 
 * distribution for details.
 *
 * libfixbuf was developed at the CERT Network Situational Awareness Group
 * by Brian Trammell <bht@cert.org> for use in the NAF and YAF tools.
 */ 

/**
 * @file
 *
 * fixbuf IPFIX protocol library public interface. Include fixbuf/public.h
 * in order to use the public fixbuf API. Calls defined in this header file 
 * will not change from version to version after fixbuf 1.0.0, and should not 
 * change aside from additions and bugfixes before fixbuf 1.0.0.
 *
 * This documentation uses IPFIX terminology as defined in RFC XXXX, 
 * "Specification of the IPFIX Protocol for the Exchange of IP Traffic Flow 
 * Information" (currently work in progress as draft-ietf-ipfix-protocol-24).
 *
 * @section types Data Types
 *
 * This file defines the data types and routines required to support IPFIX 
 * Exporting Process and IPFIX Collecting Process creation. Each data type is 
 * manipulated primarily by routines named "fb" followed by the type name 
 * (e.g., "Session", "Collector") followed by a description of the routine's 
 * action. The routines operating on the fBuf_t IPFIX Mesaage buffer type are 
 * named beginning with "fBuf". 
 *
 * The fBuf_t opaque type implements a transcoding IPFIX Message buffer for 
 * both export and collection, and is the "core" interface to the fixbuf 
 * library. 
 *
 * The fbInfoModel_t opaque type implements an IPFIX Information Model, 
 * including both IANA managed Information Elements and vendor-specific 
 * Information Elements. The fbTemplate_t opaque type implements an IPFIX 
 * Template or an IPFIX Options Template. Both are defined in terms of 
 * Information Elements, represented by the fbInfoElement_t public type. 
 * An fBuf_t message buffer maintains internal Templates, which represent 
 * records within the fixbuf application client, and external Templates, 
 * which represent records as they appear on the wire, for use during 
 * transcoding. 
 *
 * The state of an IPFIX Transport Session, including IPFIX Message Sequence 
 * Number tracking and the internal and external Templates in use within the 
 * Session, are maintained by the fbSession_t opaque type.
 *
 * An Exporting Process' connection to its corresponding Collecting Process 
 * is encapsulated by the fbExporter_t opaque type. Exporters may be created 
 * to connect via the network using one of the supported IPFIX transport 
 * protocols, or to write to IPFIX Files specified by name or by open ANSI C 
 * file pointer.
 *
 * A Collecting Process' connection to a corresponding Exporting Process is 
 * encapsulated by the fbCollector_t opaque type. The passive connection used 
 * to listen for connections from Exporting Processes is managed by the 
 * fbListener_t opaque type; Collectors can be made to read from IPFIX Files 
 * specified directly by name or by open ANSI C file pointer, as well.
 * 
 * Network addresses are specified for Exporters, Collectors, and Listeners
 * using the fbConnSpec_t and fbTransport_t public types.
 *
 * This file also defines the GError error codes used by all the fixbuf types 
 * and routines within the FB_ERROR_DOMAIN domain.
 *
 * @section export Exporter Usage
 * 
 * Each fixbuf application must have a single fbInfoModel_t instance that 
 * represents the Information Elements that the application understands. 
 * The fbInfoModelAlloc() call allocates a new Information Model with the 
 * IANA-managed information elements (current as of the fixbuf release date)
 * preloaded. Additional vendor-specific information elements may be added 
 * with fbInfoModelAddElement() and fbInfoModelAddElementArray().
 *
 * To create an Exporter, first create an fbSession_t attached to the 
 * application's fbInfoModel_t to hold the Exporter's Transport Session 
 * state using fbSessionAlloc(). Create and populate templates for addition 
 * to this session using the fbTemplate calls, then add them to the session 
 * via fbSessionAddTemplate(). 
 *
 * Note that Templates use internal reference counting, so they may be added 
 * to multiple sessions, or to the same session using multiple template IDs or 
 * multiple domains, or as both an internal and an external template on the 
 * same session.
 *
 * Then create an fbExporter_t to encapsulate the connection to the 
 * Collecting Process or the file on disk, using the fbExporterAllocFP(), 
 * fbExporterAllocFile(), or fbExporterAllocNet() calls.
 *
 * With an fbSession_t and an fbExporter_t available, create a buffer for 
 * writing via fBufAllocForExport(). Set the internal and external template 
 * IDs with fBufSetInternalTemplate() and fBufSetExportTemplate(), and use 
 * fBufAppend() to write records into IPFIX Messages and Messages to the 
 * output stream.
 *
 * By default, fBufAppend() will emit an IPFIX Message to the output stream 
 * when the end of the message buffer is reached on write. The 
 * fBufSetAutomaticMode() call can be used to modify this behavior, 
 * causing fBufAppend() to return FB_ERROR_EOM when at end of message. Use 
 * this if your application requires manual control of message export. In this 
 * case, fBufEmit() will emit a Message to the output stream.
 *
 * @section read Collector Usage - Reading from IPFIX Files
 *
 * Using fixbuf to read from IPFIX Files as a Collecting Process is very 
 * much like the Export case. Create an fbInfoModel_t and an fbSession_t as 
 * above, though you should not define external templates in the new session 
 * for collection (instead requiring them to be loaded from templates in the
 * file). 
 *
 * Then create an fbCollector_t to encapsulate the file, using the 
 * fbCollectorAllocFP() or fbCollectorAllocFile() calls.
 * 
 * With an fbSession_t and an fbCollector_t available, create a buffer for 
 * writing via fBufAllocForCollection(). Set the internal template 
 * ID with fBufSetInternalTemplate(), and use 
 * fBufNext() to read records from IPFIX Messages and Messages from the 
 * input stream.
 *
 * By default, fBufNext() will consume an IPFIX Message from the input stream 
 * when the end of the message buffer is reached on read. The 
 * fBufSetAutomaticMode() call can be used to modify this behavior, 
 * causing fBufNext() to return FB_ERROR_EOM when at end of message. Use 
 * this if your application requires manual control of message collection. 
 * In this case, fBufNextMessage() will consume a Message from the input 
 * stream.
 *
 * @section collect Collector Usage - Listening to the Network
 *
 * An additional type, fbListener_t, is used to build Collecting Processes 
 * to listen for connections from IPFIX Exporting Processes via the network. 
 * To use a listener, first create an fbInfoModel_t and an fbSession_t as 
 * above, without defining any external templates. Instead of maintaining 
 * state for a particular Transport Session, this fbSession_t instance will 
 * be used as a template for each Transport Session created by the listener.
 *
 * Then create an fbListener_t to encapsulate a passive socket on the network 
 * to wait for connections from Exporting Processes using the 
 * fbListenerAlloc() call.
 *
 * To wait for a connection from an Exporting Process, call fbListenerWait(), 
 * which handles the cloning of the fbSession_t, the creation of the 
 * fbCollector_t, and the creation of the buffer for reading from that 
 * collector, and returns the newly created fBuf_t instance. 
 *
 * Each listener tracks every active collector/buffer (i.e., each active 
 * Session) it created; the fbListenerWait() call will return an fBuf_t from 
 * which another IPFIX Message may be read if no new connections are available.
 * The preferred parameter may be used to request an fBuf_t to try first, to
 * minimize switching among available Sessions. See the documentation for 
 * fbListenerWait() for more details.
 *
 */
 
#ifndef _FB_PUBLIC_H_
#define _FB_PUBLIC_H_
#include <fixbuf/autoinc.h>

/* 
 * Error Handling Definitions
 */

/** All fixbuf errors are returned within the FB_ERROR_DOMAIN domain. */
#define FB_ERROR_DOMAIN             g_quark_from_string("fixbufError")
/** No template was available for the given template ID. */
#define FB_ERROR_TMPL               1
/** 
 * End of IPFIX message. Either there are no more records present in the
 * message on read, or the message MTU has been reached on write. 
 */
#define FB_ERROR_EOM                2
/** 
 * End of IPFIX Message stream. No more messages are available from the
 * transport layer on read, either because the session has closed, or the
 * file has been processed.
 */
#define FB_ERROR_EOF                3
/** 
 * Illegal IPFIX mesaage content on read. The input stream is malformed, or 
 * is not an IPFIX Message after all.
 */
#define FB_ERROR_IPFIX              4
/** 
 * A message was received larger than the collector buffer size. 
 * Should never occur. This condition is checked at the transport layer
 * in case future versions of fixbuf support dynamic buffer sizing. 
 */
#define FB_ERROR_BUFSZ              5
/** The requested feature is not yet implemented. */
#define FB_ERROR_IMPL               6
/** An unspecified I/O error occured. */
#define FB_ERROR_IO                 7
/** 
 * No data is available for reading from the transport layer. 
 * Either a transport layer read was interrupted, or timed out.
 */
#define FB_ERROR_NLREAD             8
/** 
 * An attempt to write data to the transport layer failed due to
 * closure of the remote end of the connection. Currently only occurs with
 * the TCP transport layer.
 */
#define FB_ERROR_NLWRITE            9
/**
 * The specified Information Element does not exist in the Information Model.
 */
#define FB_ERROR_NOELEMENT          10
/**
 * A connection or association could not be established or maintained.
 */
#define FB_ERROR_CONN               11

/*
 * Public Datatypes and Constants
 */
 
struct fBuf_st;
/** 
 * An IPFIX message buffer. Used to encode and decode records from 
 * IPFIX Messages. The internals of this structure are private to 
 * libfixbuf. 
 */
typedef struct fBuf_st fBuf_t;

/**
 * A variable-length field value. Variable-length information element 
 * content is represented by an fbVarfield_t on the internal side of the 
 * transcoder; that is, variable length fields in an IPFIX Message must be
 * represented by this structure within the application record.
 */
typedef struct fbVarfield_st {
    /** Length of content in buffer. */
    size_t      len;
    /** 
     * Content buffer. In network byte order as appropriate. On write, this 
     * buffer will be copied into the message buffer. On read, this buffer 
     * points into the message buffer and must be copied by the caller before
     * any call to fBufNext(). 
     */
    uint8_t     *buf;
} fbVarfield_t;

struct fbInfoModel_st;
/** 
 * An IPFIX information model. Contains information element definitions. 
 * The internals of this structure are private to libfixbuf. 
 */
typedef struct fbInfoModel_st fbInfoModel_t;

/**
 * Convenience macro for creating fbInfoElement_t static initializers.
 * Used for creating information element arrays suitable for passing to
 * fbInfoModelAddElementArray().
 */
#define FB_IE_INIT(_name_, _ent_, _num_, _len_, _flags_) \
    { {.name = _name_}, 0, _ent_, _num_, _len_, _flags_ }

/**
 * Convenience macro defining a null information element initializer to 
 * terminate a constant information element array for passing to 
 * fbInfoModelAddElementArray().
 */
#define FB_IE_NULL \
    { {.name = NULL}, 0, 0, 0, 0, 0 }

/** 
 * Default treatment flags value. Provided for initializer convenience.
 * Corresponds to octet-array semantics for a non-reversible, non-alien IE.
 */
#define FB_IE_F_NONE                            0x00000000

/**
 * Information element endian conversion flag. If set, IE is an integer and
 * will be endian-converted on transcode.
 */
#define FB_IE_F_ENDIAN                          0x00000001

/**
 * Information element reversible flag. If set for an information element
 * with an enterprise number of 0 (an IETF/IANA IE), adding the information
 * element via fbInfoModelAddElement() or fbInfoModelAddElementArray() will
 * cause a second, reverse information element to be added to the model 
 * following the conventions in IETF Internet-Draft draft-ietf-ipfix-biflow-03.
 * Note that the reverse PEN has not yet been assigned, so this implementation
 * uses a provisional reverse IE as defined by the macro FB_IE_PEN_REVERSE.
 */
#define FB_IE_F_REVERSIBLE                      0x00000040

/**
 * Information element alien flag. If set, IE is enterprise-specific and was
 * recieved via an external template at a Collecting Process. It is therefore 
 * subject to semantic typing via options (not yet implemented). Do not set this
 * flag on information elements added programmatically to an information model 
 * via fbInfoModelAddElement() or fbInfoModelAddElementArray().
 */
#define FB_IE_F_ALIEN                           0x00000080

/**
 * Information element length constant for variable-length IE.
 */
#define FB_IE_VARLEN                            65535

/**
 * Private enterprise number for reverse information elements
 * (see draft-ietf-ipfix-biflow-03 section 6.1).  If an information element with 
 * FB_IE_F_REVERSIBLE and a zero enterprise number (i.e., an IANA-assigned 
 * information element) is added to a model, the reverse IE will be generated 
 * by setting the enterprise number to this constant. 
 */
#define FB_IE_PEN_REVERSE                       29305

/**
 * Reverse information element bit for vendor-specific information elements
 * (see draft-ietf-ipfix-biflow-03 section 6.2). If an information element with 
 * FB_IE_F_REVERSIBLE and a non-zero enterprise number (i.e., a vendor-specific
 * information element) is added to a model, the reverse IE number will be 
 * generated by ORing this bit with the given forward information element 
 * number. 
 */
#define FB_IE_VENDOR_BIT_REVERSE                0x4000

/** 
 * Reverse information element name prefix. This string is prepended to an
 * information element name, and the first character after this string
 * is capitalized, when generating a reverse information element.
 */
#define FB_IE_REVERSE_STR                       "reverse"

/** Length of reverse information element name prefix. */
#define FB_IE_REVERSE_STRLEN                    7

/** 
 * A single IPFIX Information Element definition. 
 * An Information Element defines the type of data in each field of
 * a record. This structure may be contained in an fbInfoModel_t,
 * in which case the name field contians the information element name,
 * or an an fbTemplate_t, in which case the canon field references the
 * fbInfoElement_t contained within the Information Model.
 */
typedef struct fbInfoElement_st {
    /** Information element name. */
    union {
        /** 
         * Pointer to canonical copy of IE. 
         * Set by fbInfoElementCopyToTemplate(), 
         * and valid only for template IEs. 
         */
        const struct fbInfoElement_st *canon;
        /** 
         * Information element name. Storage for this is managed
         * by fbInfoModel. Valid only for model IEs.
         */
        char                    *name;
    } 
                        ref;
    /** 
     * Multiple IE index. Must be 0 for model IEs. 
     * Defines the ordering of identical IEs in templates.
     * Set and managed automatically by the fbTemplate_t routines.
     */ 
    uint32_t            midx;
    /** Private Enterprise Number. Set to 0 for IETF-defined IEs. */
    uint32_t            ent;
    /** 
     * Information Element number. Does not include the on-wire 
     * enterprise bit; i.e. num & 0x8000 == 0 even if ent > 0. 
     */
    uint16_t            num;
    /** Information element length in octets. */
    uint16_t            len;
    /** Flags. Bitwise OR of FB_IE_F_* constants. */
    uint32_t            flags;
} fbInfoElement_t;

/** 
 * Template ID argument to pass to fbSessionAddTemplate to automatically 
 * assign a template ID.
 */
#define FB_TID_AUTO         0

/**
 * Reserved set ID for template sets.
 */
#define FB_TID_TS           2

/**
 * Reserved set ID for options template sets.
 */
#define FB_TID_OTS          3

/**
 * Minimum non-reserved template ID available for data sets.
 */
#define FB_TID_MIN_DATA     256

struct fbTemplate_st;
/** 
 * An IPFIX Template or Options Template. Templates define the structure of 
 * data records and options records within an IPFIX Message.
 * The internals of this structure are private to libfixbuf. 
 */
typedef struct fbTemplate_st fbTemplate_t;

/**
 * Convenience macro defining a null information element specification 
 * initializer to terminate a constant information element specifier array 
 * for passing to fbTemplateAppendSpecArray().
 */
#define FB_IESPEC_NULL { NULL, 0, 0 }

/** 
 * A single IPFIX Information Element specification.
 * Used to name an information element for inclusion in a template by 
 * fbTemplateAppendSpecArray().
 */
typedef struct fbInfoElementSpec_st {
    /** Information element name */
    char                *name;
    /** 
     * Length override; if nonzero, replace the length of the IE from the
     * model with this length. Used for reduced-length encoding.
     */
    uint16_t            len_override;
    /**
     * Application flags word. If nonzero, then the flags argument to 
     * fbTemplateAppendSpec() or fbTemplateAppendSpecArray() MUST match at
     * least one bit of this flags word in order for the information element 
     * to be appended.
     */
    uint32_t            flags;
} fbInfoElementSpec_t;

struct fbSession_st;
/** 
 * An IPFIX Transport Session state container. Though Session creation and 
 * lifetime are managed by the fbCollector_t and fbExporter_t types, each 
 * fBuf_t buffer uses this type to store session state, including internal 
 * and external Templates and Message Sequence Number information.
 */
typedef struct fbSession_st fbSession_t;

/** Transport protocol for connection specifier. */
typedef enum fbTransport_en {
    /** 
     * Partially reliable datagram transport via SCTP. 
     * Only available if fixbuf was built with SCTP support. 
     */
    FB_SCTP,
    /** Reliable stream transport via TCP. */
    FB_TCP,
    /** Unreliable datagram transport via UDP. */
    FB_UDP,
    /** 
     * Secure, partially reliable datagram transport via DTLS over SCTP. 
     * Only available if fixbuf was built with OpenSSL support.
     * Requires an OpenSSL implementation of DLTS over SCTP, not yet available.
     */
    FB_DTLS_SCTP,
    /** 
     * Secure, reliable stream transport via TLS over TCP.
     * Only available if fixbuf was built with OpenSSL support.
     */
    FB_TLS_TCP,
    /** 
     * Secure, unreliable datagram transport via DTLS over UDP.
     * Only available if fixbuf was built with OpenSSL support.
     * Requires OpenSSL 0.9.8 or later with DTLS support.
     */
    FB_DTLS_UDP,
} fbTransport_t;

/** 
 * Connection specifier. Used to define a peer address for fbExporter_t, or a
 * passive address for fbListener_t.
 */
typedef struct fbConnSpec_st {
    /** Transport protocol to use */
    fbTransport_t       transport;
    /** Hostname to connect/listen to. NULL to listen on all interfaces. */
    char                *host;
    /** Service name or port number to connect/listen to. */
    char                *svc;
    /** Path to certificate authority file. Only used for OpenSSL transport. */
    char                *ssl_ca_file;
    /** Path to certificate file. Only used for OpenSSL transport. */
    char                *ssl_cert_file;
    /** Path to private key file. Only used for OpenSSL transport. */
    char                *ssl_key_file;
    /** Private key decryption password. Only used for OpenSSL transport. */
    char                *ssl_key_pass;
    /** 
     * Pointer to address info cache. Initialize to NULL. 
     * For fixbuf internal use only.
     */
    void                *vai;
    /** 
     * Pointer to SSL context cache. Initialize to NULL. 
     * For fixbuf internal use only.
     */
    void                *vssl_ctx;
} fbConnSpec_t;

/**
 * Convenience macro defining a null static fbConnSpec_t.
 */
#define FB_CONNSPEC_INIT { FB_SCTP, NULL, NULL,         \
                           NULL, NULL, NULL, NULL,      \
                           NULL, NULL }

struct fbExporter_st;
/**
 * IPFIX Exporting Process endpoint. Used to export messages from an associated
 * IPFIX Message Buffer to a remote Collecting Process, or to an IPFIX File.
 * The internals of this structure are private to libfixbuf. 
 */
typedef struct fbExporter_st fbExporter_t;

struct fbCollector_st;
/**
 * IPFIX Collecting Process endpoint. Used to collect messages into an 
 * associated IPFIX Message Buffer from a remote Exporting Process, or from 
 * an IPFIX File. Use this with the fbListener_t structure to implement a full 
 * Collecting Process, including Transport Session setup. The internals of 
 * this structure are private to libfixbuf. 
 */
typedef struct fbCollector_st fbCollector_t;

struct fbListener_st;
/** 
 * IPFIX Collecting Process session listener. Used to wait for connections
 * from IPFIX Exporting Processes, and to manage open connections via a 
 * select(2)-based mechanism. The internals of this structure are private 
 * to libfixbuf. 
 */
typedef struct fbListener_st fbListener_t;
 
/** 
 * Application context initialization function type for fbListener_t.
 * This function is called after accept(2) for TCP or SCTP with the peer
 * address in the peer argument. For UDP, it is called during fbListener_t
 * initialization. The application may veto fbCollector_t creation by
 * returning FALSE. The context (returned via out-parameter ctx) will be
 * stored in the fbCollector_t, and is retrievable via a call to 
 * fbCollectorGetContext().
 */
typedef gboolean        (*fbListenerAppInit_fn) (
    fbListener_t                *listener,
    void                        **ctx,
    int                         fd,
    struct sockaddr             *peer,
    size_t                      peerlen,
    GError                      **err);

/** 
 * Application context free function type for fbListener_t.
 * Called during fbCollector_t cleanup.
 */
typedef void            (*fbListenerAppFree_fn) (
    void                        *ctx);

/*
 * Public Function Calls. These calls will remain available and retain 
 * their functionality in all subsequent versions of libfixbuf.
 */


/**
 * Set the internal template on a buffer to the given template ID. The internal 
 * template describes the format of the record pointed to by the recbase 
 * parameter to fBufAppend() (for export) and fBufNext() (for collection). The
 * given template ID must identify a current internal template in the buffer's 
 * associated session.
 *
 * An internal template must be set on a buffer before calling fBufAppend() or 
 * fBufNext().
 *
 * @param fbuf      an IPFIX message buffer
 * @param int_tid   template ID of the new internal template
 * @param err       An error description, set on failure.
 * @return TRUE on success, FALSE on failure.
 */
  
gboolean            fBufSetInternalTemplate(
    fBuf_t              *fbuf,
    uint16_t            int_tid,
    GError              **err);

/**
 * Set the external template for export on a buffer to the given template ID. 
 * The external template describes the record that will be written to the IPFIX 
 * message. The buffer must be initialized for export. The given ID is scoped 
 * to the observation domain of the associated session 
 * (see fbSessionSetDomain()), and must identify a current external template 
 * for the current domain in the buffer's associated session. 
 *
 * An export template must be set on a buffer before calling fBufAppend().
 *
 * @param fbuf      an IPFIX message buffer
 * @param ext_tid   template ID of the new external template within the 
 *                  current domain.
 * @param err       An error description, set on failure.
 * @return TRUE on success, FALSE on failure.
 */

gboolean            fBufSetExportTemplate(
    fBuf_t              *fbuf,
    uint16_t            ext_tid,
    GError              **err);

/**
 * Set the automatic mode flag on a buffer. In automatic mode, a call to 
 * fBufAppend() or fbSessionExportTemplates() that overruns the available space 
 * in the buffer will cause a call to fBufEmit() to emit the message in the 
 * buffer to the exporter before starting a new message; and a call to 
 * fBufNext() that overruns the buffer will cause a call to fBufNextMessage() 
 * to read another message from the collector before attempting to read a 
 * record. In manual mode, end of message on any buffer read/write call
 * results in FB_ERROR_EOM. Buffers are created in automatic mode by default.
 *
 * @param fbuf      an IPFIX message buffer
 * @param automatic TRUE for this buffer to be automatic, FALSE for manual.
 */

void                fBufSetAutomaticMode(
    fBuf_t              *fbuf,
    gboolean            automatic);

/**
 * Retrieve the session associated with a buffer.
 *
 * @param fbuf      an IPFIX message buffer
 * @return the associated session
 */
 
fbSession_t         *fBufGetSession(
    fBuf_t              *fbuf);

/**
 * Free a buffer. Also frees any associated session, exporter, or collector, 
 * closing exporting process or collecting process endpoint connections
 * and removing collecting process endpoints from any listeners, as necessary.
 *
 * @param fbuf      an IPFIX message buffer
 */
     
void                fBufFree(
    fBuf_t              *fbuf);

/**
 * Allocate a new buffer for export. Associates the buffer with a given 
 * session and exporting process endpoint; these become owned by the buffer.
 *
 * @param session   a session initialized with appropriate 
 *                  internal and external templates 
 * @param exporter  an exporting process endpoint
 * @return a new IPFIX message buffer, owning the session and exporter,
 *         for export use via fBufAppend() and fBufEmit().
 */

fBuf_t              *fBufAllocForExport(
    fbSession_t         *session,
    fbExporter_t        *exporter);

/**
 * Retrieve the exporting process endpoint associated with a buffer.
 * The buffer must have been allocated with fBufAllocForExport(); 
 * otherwise, returns NULL.
 * 
 * @param fbuf      an IPFIX message buffer
 * @return the associated exporting process endpoint
 */

fbExporter_t        *fBufGetExporter(
    fBuf_t              *fbuf);

/**
 * Associate an exporting process endpoint with a buffer. 
 * The exporter will be used to write IPFIX messgaes to a transport. 
 * The exporter becomes owned by the buffer; any previous exporter 
 * associated with the buffer is closed if necessary and freed.
 *
 * @param fbuf      an IPFIX message buffer
 * @param exporter  an exporting process endpoint
 */
 
void                fBufSetExporter(
    fBuf_t              *fbuf,
    fbExporter_t        *exporter);

/**
 * Append a record to a buffer. Uses the present internal template set via 
 * fBufSetInternalTemplate() to describe the record of size recsize located 
 * in memory at recbase.  Uses the present export template set via 
 * fBufSetExportTemplate() to describe the record structure to be written to 
 * the buffer. Information Elements present in the external template that are 
 * not present in the internal template are transcoded into the message as 
 * zeroes. If the buffer is in automatic mode, may cause a message to be 
 * emitted via fBufEmit() if there is insufficient space in the buffer for 
 * the record.
 *
 * If the internal template contains any variable length Information Elements, 
 * those must be represented in the record by fbVarfield_t structures. 
 *
 * @param fbuf      an IPFIX message buffer
 * @param recbase   pointer to internal record
 * @param recsize   size of internal record in bytes
 * @param err       an error description, set on failure. 
 *                  Must not be NULL, as it is used internally in 
 *                  automatic mode to detect message restart.
 * @return TRUE on success, FALSE on failure.
 */
 
gboolean            fBufAppend(
    fBuf_t              *fbuf,
    uint8_t             *recbase,
    size_t              recsize,
    GError              **err);

/**
 * Emit the message currently in a buffer using the associated exporting 
 * process endpoint. 
 *
 * @param fbuf      an IPFIX message buffer
 * @param err       an error description, set on failure. 
 * @return TRUE on success, FALSE on failure.
 */

gboolean            fBufEmit(
    fBuf_t              *fbuf,
    GError              **err);

/**
 * Set the export time on the message currently in a buffer. This will be used
 * as the export time of the message created by the first call to fBufAppend()
 * after the current message, if any, is emitted. Use 0 for the export time
 * to cause the export time to be taken from the system clock at message 
 * creation time.
 *
 * @param fbuf      an IPFIX message buffer
 * @param extime    the export time in epoch seconds.
 */

void                fBufSetExportTime(
    fBuf_t              *fbuf,
    uint32_t            extime);

/**
 * Allocate a new buffer for collection. Associates the buffer with a given 
 * session and collecting process endpoint; these become owned by the buffer.
 *
 * @param session   a session initialized with appropriate 
 *                  internal templates 
 * @param collector  an collecting process endpoint
 * @return a new IPFIX message buffer, owning the session and collector,
 *         for collection use via fBufNext() and fBufNextMessage().
 */

fBuf_t              *fBufAllocForCollection(
    fbSession_t         *session,
    fbCollector_t       *collector);
    
/**
 * Retrieve the collecting process endpoint associated with a buffer.
 * The buffer must have been allocated with fBufAllocForCollection(); 
 * otherwise, returns NULL.
 * 
 * @param fbuf      an IPFIX message buffer
 * @return the associated collecting process endpoint
 */
 
fbCollector_t       *fBufGetCollector(
    fBuf_t              *fbuf);

/**
 * Associate an collecting process endpoint with a buffer. 
 * The collector will be used to read IPFIX messgaes from a transport. 
 * The collector becomes owned by the buffer; any previous collector 
 * associated with the buffer is closed if necessary and freed.
 *
 * @param fbuf      an IPFIX message buffer
 * @param collector  an collecting process endpoint
 */
 
void                fBufSetCollector(
    fBuf_t              *fbuf,
    fbCollector_t       *collector);

/**
 * Retrieve a record from a buffer. Uses the external template taken from
 * the message to read the next record available from a data set in the message. 
 * Copies the record to a buffer at recbase, with a maximum record size 
 * pointed to by recsize, described by the present internal template set via 
 * fBufSetInternalTemplate(). Reads and processes any templates and options
 * templates between the last record read (or beginning of message) and the
 * next data record. Information Elements present in the internal template 
 * that are not present in the external template are transcoded into the 
 * record at recbase as zeroes. If the buffer is in automatic mode, may cause 
 * a message to be read via fBufNextMessage() if there are no more records 
 * available in the message buffer.
 *
 * If the internal template contains any variable length Information Elements, 
 * those must be represented in the record at recbase by fbVarfield_t 
 * structures. 
 *
 * @param fbuf      an IPFIX message buffer
 * @param recbase   pointer to internal record buffer; will contain
 *                  record data after call.
 * @param recsize   On call, pointer to size of internal record buffer
 *                  in bytes. Contains number of bytes actually transcoded
 *                  at end of call.
 * @param err       an error description, set on failure. 
 *                  Must not be NULL, as it is used internally in 
 *                  automatic mode to detect message restart.
 * @return TRUE on success, FALSE on failure.
 */
 
gboolean            fBufNext(
    fBuf_t              *fbuf,
    uint8_t             *recbase,
    size_t              *recsize,
    GError              **err);

/**
 * Read a new message into a buffer using the associated collecting 
 * process endpoint. Called by fBufNext() on end of message in automatic
 * mode; should be called after an FB_ERROR_EOM return from fBufNext in
 * manual mode, or to skip the current message and go on to the next
 * in the stream.
 *
 * @param fbuf      an IPFIX message buffer
 * @param err       an error description, set on failure. 
 * @return TRUE on success, FALSE on failure.
 */


gboolean            fBufNextMessage(
    fBuf_t              *fbuf,
    GError              **err);
    
/**
 * Retrieve the export time on the message currently in a buffer. 
 *
 * @param fbuf      an IPFIX message buffer
 * @return the export time in epoch seconds.
 */

uint32_t            fBufGetExportTime(
    fBuf_t              *fbuf);

/**
 * Retrieve the external template used to read the last record from the buffer.
 * If no record has been read, returns NULL. Stores the external template ID
 * within the current domain in ext_tid, if not NULL.
 *
 * This routine is not particularly useful to applications, as it would be
 * called after the record described by the external template had been 
 * transcoded, and as such could not be used to select an 
 * appropriate internal template for a given external template. However,
 * it is used by fBufNextCollectionTemplate(), and may be useful in certain
 * contexts, so is made public.
 *
 * Usually, you'll want to use fBufNextCollectionTemplate() instead.
 *
 * @param fbuf      an IPFIX message buffer
 * @param ext_tid   pointer to external template ID storage, or NULL.
 * @return the external template describing the last record read. 
 */
 
fbTemplate_t    *fBufGetCollectionTemplate(
    fBuf_t          *fbuf,
    uint16_t        *ext_tid);

/**
 * Retrieve the external template that will be used to read the next record 
 * from the buffer. If no next record is available, returns NULL. Stores the 
 * external template ID within the current domain in ext_tid, if not NULL.
 * Reads and processes any templates and options
 * templates between the last record read (or beginning of message) and the
 * next data record. If the buffer is in automatic mode, may cause 
 * a message to be read via fBufNextMessage() if there are no more records 
 * available in the message buffer.
 * 
 * @param fbuf      an IPFIX message buffer
 * @param ext_tid   pointer to external template ID storage, or NULL.
 * @param err       an error description, set on failure. 
 *                  Must not be NULL, as it is used internally in 
 *                  automatic mode to detect message restart.
 * @return the external template describing the last record read. 
 */

fbTemplate_t    *fBufNextCollectionTemplate(
    fBuf_t          *fbuf,
    uint16_t        *ext_tid,
    GError          **err);

/**
 * Allocate a new information model. The information model will contain all
 * the default information elements in the IANA-managed number space, and may
 * be extended via fbInfoModelAddElement() and fbInfoModelAddElementArray().
 *
 * An Information Model is required to create Templates and Sessions. Each
 * application should have only one Information Model.
 *
 * @return a new Information Model
 */ 

fbInfoModel_t       *fbInfoModelAlloc();

/**
 * Free an information model. Must not be called until all sessions and
 * templates depending on the information model have also been freed; i.e.,
 * at application cleanup time.
 *
 * @param model     An information model
 */

void                fbInfoModelFree(
    fbInfoModel_t       *model);

/** 
 * Add a single information element to an information
 * model. The information element is assumed to be in "canonical" form; that 
 * is, its ref.name field should contain the information element name. The 
 * information element and its name are copied into the model; the caller may 
 * free or reuse its storage after this call.
 *
 * See fbInfoModelAddElementArray() for a more convenient method of statically
 * adding information elements to information models.
 *
 * @param model     An information model
 * @param ie        Pointer to an information element to copy into the model
 */
 
void                fbInfoModelAddElement(
    fbInfoModel_t       *model,
    fbInfoElement_t     *ie);

/**
 * Add multiple information elements in an array to an information 
 * model. The information elements are assumed to be in "canonical" form; that 
 * is, their ref.name fields should contain the information element name. Each 
 * information element and its name are copied into the model; the caller may 
 * free or reuse its storage after this call.
 *
 * The ie parameter points to the first information element in an array,
 * usually statically initialized with an array of FB_IE_INIT macros followed
 * by an FB_IE_NULL macro.
 * @param model     An information model
 * @param ie        Pointer to an IE array to copy into the model
 */
    
void                fbInfoModelAddElementArray(
    fbInfoModel_t       *model,
    fbInfoElement_t     *ie);

/**
 * Return a pointer to the canonical information element within an information
 * model given the information element name. The returned information element
 * is owned by the information model and must not be modified.
 * 
 * @param model     An information model
 * @param name      The name of the information element to look up
 * @return          The named information element within the model, 
 *                  or NULL if no such element exists.
 */

const fbInfoElement_t     *fbInfoModelGetElementByName(
    fbInfoModel_t       *model,
    const char          *name);


/**
 * Allocate a new empty template. The template will be associated with the 
 * given Information Model, and only able to use Information Elements defined
 * within that Information Model. Templates may be associated with multiple
 * sessions, with different template IDs each time, and as such are
 * reference counted and owned by sessions. A template must be associated
 * with at least one session or it will be leaked; each template is freed
 * after its last associated session is freed.
 * 
 * Use fbTemplateAppend(), fbTemplateAppendSpec(), and 
 * fbTemplateAppendSpecArray() to "fill in" a template after creating it,
 * and before associating it with any session.
 *
 * @param model     An information model
 * @return a new, empty Template.
 */
 
fbTemplate_t        *fbTemplateAlloc(
    fbInfoModel_t       *model);

/**
 * Append an information element to a template. The information element is taken
 * to be an example; the canonical element from the template's associated model 
 * is looked up by enterprise and element number and copied. If no information
 * element exists in the model with the given enterprise and element number,
 * it is copied to the model with the name "_alienInformationElement".
 *
 * This call is intended primarily for use by fBuf_t's template reader, but can
 * also be useful to simulate receipt of templates over the wire.
 *
 * @param tmpl      Template to append information element to
 * @param ex_ie     Example IE to add to the template
 * @param err       an error description, set on failure. 
 * @return TRUE on success, FALSE on failure.
 */
 
gboolean            fbTemplateAppend(
    fbTemplate_t        *tmpl,
    fbInfoElement_t     *ex_ie,
    GError              **err);

/**
 * Append an information element described by specifier to a template. 
 * The information element named by the specifier is copied from the template's
 * associated model, and the length and flags are overriden from the specifier.
 *
 * @param tmpl      Template to append information element to.
 * @param spec      Specifier describing information element to append.
 * @param flags     Application flags. Must match one bit of spec flags word
 *                  or the append will be silently skipped. Used for
 *                  building multiple templates with different information
 *                  element features from a single specifier.
 * @param err       an error description, set on failure. 
 * @return TRUE on success, FALSE on failure.
 */

gboolean            fbTemplateAppendSpec(
    fbTemplate_t        *tmpl,
    fbInfoElementSpec_t *spec,
    uint32_t            flags,
    GError              **err);

/**
 * Append information elements described by a specifier array to a template. 
 * The information elements named by the specifiers are copied from the 
 * template's associated model, and the length and flags are overriden from
 * each specifier. The array is read until the FB_IESPEC_NULL convenience macro
 * is encountered.
 *
 * @param tmpl      Template to append information element to.
 * @param spec      Pointer to first specifier in specifier array to append.
 * @param flags     Application flags. Must contain all bits of spec flags word
 *                  or the append will be silently skipped. Used for
 *                  building multiple templates with different information
 *                  element features from a single specifier.
 * @param err       an error description, set on failure. 
 * @return TRUE on success, FALSE on failure.
 */

gboolean            fbTemplateAppendSpecArray(
    fbTemplate_t        *tmpl,
    fbInfoElementSpec_t *spec,
    uint32_t            flags,
    GError              **err);

/**
 * Determine number of information elements in a template.
 *
 * @param tmpl      A template
 * @return information element count
 */
 
uint32_t            fbTemplateCountElements(
    fbTemplate_t        *tmpl);

/**
 * Set the number of information elements in a template that are scope. This
 * causes the template to become an options template, and must be called after
 * all the scope information elements have been appended to the template.
 *
 * @param tmpl          Template to set scope on
 * @param scope_count   Number of scope information elements
 */

void                fbTemplateSetOptionsScope(
    fbTemplate_t        *tmpl,
    uint16_t            scope_count);

/**
 * Determine number of scope information elements in a template. The template
 * is an options template if nonzero.
 *
 * @param tmpl      A template
 * @return scope information element count
 */
uint32_t            fbTemplateGetOptionsScope(
    fbTemplate_t        *tmpl);

/**
 * Determine if a template contains a given information element. Matches against
 * information element private enterprise number, number, and multiple-IE index
 * (i.e., to determine if a given template contains six instances of a given
 * information element, set ex_ie->midx = 5 before this call).
 *
 * @param tmpl      Template to search
 * @param ex_ie     Pointer to an information element to search for 
 * @return          TRUE if the template contains the given IE
 */

gboolean           fbTemplateContainsElement(
    fbTemplate_t            *tmpl,
    const fbInfoElement_t   *ex_ie);

/**
 * Determine if a template contains at least one instance of a given 
 * information element, specified by name in the template's information model.  
 *
 * @param tmpl      Template to search
 * @param spec      Specifier of information element to search for 
 * @return          TRUE if the template contains the given IE
 */

gboolean           fbTemplateContainsElementByName(
    fbTemplate_t        *tmpl,
    fbInfoElementSpec_t *spec);

/**
 * Determine if a template contains at least one instance of each 
 * information element in a given information element specifier array.
 *
 * @param tmpl      Template to search
 * @param spec      Pointer to specifier array to search for 
 * @return          TRUE if the template contains all the given IEs
 */
 
gboolean           fbTemplateContainsAllElementsByName(
    fbTemplate_t        *tmpl,
    fbInfoElementSpec_t *spec);

/**
 * Free a template if it is not currently in use by any Session. Use this
 * to clean up while creating templates in case of error.
 *
 * @param tmpl template to free
 */
 
void                fbTemplateFreeUnused(
    fbTemplate_t        *tmpl);

/**
 * Allocate a transport session state container. The new session is associated
 * with the given information model, contains no templates, and is usable 
 * either for collection or export.
 *
 * Each fbExporter_t, fbListener_t, and fbCollector_t must have its own session;
 * session state cannot be shared.
 *
 * @param model     An information model
 * @return a new, empty session state container.
 */
 
fbSession_t         *fbSessionAlloc(
    fbInfoModel_t       *model);

/**
 * Free a transport session state container. This is done automatically when 
 * freeing the listener or buffer with which the session is
 * associated. Use this call if a session needs to be destroyed before it
 * is associated.
 *
 * @param session   session state container to free.
 */

void                fbSessionFree(
    fbSession_t         *session);

/**
 * Reset the external state (sequence numbers and templates) in a session
 * state container. 
 *
 * FIXME: Verify that this call actually makes sense; either that a session
 * is reassociatable with a new collector, or that you need to do this when
 * reassociating a collector with a connection. Once this is done, rewrite
 * this documentation
 *
 * @param session   session state container to reset
 */

void                fbSessionResetExternal(
    fbSession_t         *session);

/**
 * Set the current observation domain on a session. The domain
 * is used to scope sequence numbers and external templates. This is called
 * automatically during collection, but must be called to set the domain
 * for export before adding external templates or writing records.
 *
 * Notice that a domain change does not automatically cause any associated
 * export buffers to emit messages; a domain change takes effect with the
 * next message started. Therefore, call fBufEmit() before setting the domain
 * on the buffer's associated session.
 *
 * @param session   a session state container
 * @param domain    ID of the observation domain to set
 */

void                fbSessionSetDomain(
    fbSession_t         *session,
    uint32_t            domain);

/**
 * Retrieve the current domain on a session.
 *
 * @param session a session state container
 * @return the ID of the session's current observation domain
 */

uint32_t            fbSessionGetDomain(
    fbSession_t         *session);

/**
 * Export a single external template in the current domain of a given session.
 * Writes the template to the associated export buffer. May cause a message to
 * be emitted if the associated export buffer is in automatic mode, or return
 * with FB_ERROR_EOM if the associated export buffer is not in automatic mode. 
 *
 * @param session   a session state container associated with an export buffer
 * @param tid       template ID within current domain to export
 * @param err       an error description, set on failure. 
 * @return TRUE on success, FALSE on failure.
 */

gboolean            fbSessionExportTemplate(
    fbSession_t         *session,
    uint16_t            tid,
    GError              **err);

/**
 * Export all external templates in the current domain of a given session.
 * Writes templates to the associated export buffer. May cause a message to
 * be emitted if the associated export buffer is in automatic mode, or return
 * with FB_ERROR_EOM if the associated export buffer is not in automatic mode. 
 *
 * @param session   a session state container associated with an export buffer
 * @param err       an error description, set on failure. 
 * @return TRUE on success, FALSE on failure.
 */

gboolean            fbSessionExportTemplates(
    fbSession_t         *session,
    GError              **err);

/**
 * Add a template to a session. If external, adds the template to the current
 * domain, and exports the template if the session is associated with an export
 * buffer. Assigns the template ID given in tid, or assigns a template ID if
 * tid is FB_TID_AUTO. 
 *
 * @param session   A session state container
 * @param internal  TRUE if the template is internal, FALSE if external.
 * @param tid       Template ID to assign, replacing any current template
 *                  in case of collision; or FB_TID_AUTO to assign a new tId.
 * @param tmpl      Template to add
 * @param err       An error description, set on failure
 * @return the template ID of the added template, or 0 on failure.
 */

uint16_t            fbSessionAddTemplate(
    fbSession_t         *session,
    gboolean            internal,
    uint16_t            tid,
    fbTemplate_t        *tmpl,
    GError              **err);

/** 
 * Remove a template from a session.  If external, removes the template from 
 * the current domain, and exports a template revocation set if the session is 
 * associated with an export buffer. 
 *
 * @param session   A session state container
 * @param internal  TRUE if the template is internal, FALSE if external.
 * @param tid       Template ID to remove.
 * @param err       An error description, set on failure
 * @return TRUE on success, FALSE on failure.
 */

gboolean            fbSessionRemoveTemplate(
    fbSession_t         *session,
    gboolean            internal,
    uint16_t            tid,
    GError              **err);

/**
 * Retrieve a template from a session by ID. If external, retrieves the 
 * template within the current domain. 
 *
 * @param session   A session state container
 * @param internal  TRUE if the template is internal, FALSE if external.
 * @param tid       ID of the template to retrieve.
 * @param err       An error description, set on failure.
 * @return The template with the given ID, or NULL on failure.
 */

fbTemplate_t        *fbSessionGetTemplate(
    fbSession_t         *session,
    gboolean            internal,
    uint16_t            tid,
    GError              **err);

/**
 * Allocate an exporting process endpoint for a network connection. 
 * The remote collecting process is specified by the given connection specifier.
 * The underlying socket connection will not be opened until the first message 
 * is emitted from the buffer associated with the exporter.
 *
 * @param spec      remote endpoint connection specifier
 * @return a new exporting process endpoint
 */

fbExporter_t        *fbExporterAllocNet(
    fbConnSpec_t        *spec);

/**
 * Allocate an exporting process endpoint for a named file. The underlying
 * file will not be opened until the first message is emitted from the
 * buffer associated with the exporter.
 *
 * @param path      pathname of the IPFIX File to write, or "-" to
 *                  open standard output.
 * @return a new exporting process endpoint
 */
 
fbExporter_t        *fbExporterAllocFile(
    const char          *path);

/**
 * Allocate an exporting process endpoint for an opened ANSI C file pointer.
 *
 * @param fp        open file pointer to write to
 * @return a new exporting process endpoint
 */
 
fbExporter_t        *fbExporterAllocFP(
    FILE                *fp);
    
/**
 * Set the SCTP stream for the next message exported. To change the SCTP
 * stream used for export, first emit any message in the exporter's associated 
 * buffer with fbufEmit(), then use this call to set the stream for the next 
 * message. This call cancels automatic stream selection, use 
 * fbExporterAutoStream() to re-enable it. This call is a no-op for non-SCTP 
 * exporters.
 *
 * @param exporter      an exporting process endpoint.
 * @param sctp_stream   SCTP stream to use for next message.
 */
 
void                fbExporterSetStream(
    fbExporter_t        *exporter,
    int                 sctp_stream);

/**
 * Enable automatic SCTP stream selection for the next message exported. 
 * Automatic stream selection is the default; use this call to re-enable it
 * on a given exporter after using fbExporterSetStream(). With automatic 
 * stream selection, the minimal behavior specified in the original IPFIX
 * protocol (RFC xxxx) is used: all templates and options templates are
 * exported on stream 0, and all data is exported on stream 1. This call is a 
 * no-op for non-SCTP exporters.
 *
 * @param exporter      an exporting process endpoint.
 */
 
void                fbExporterAutoStream(
    fbExporter_t        *exporter);

/**
 * Force the file or socket underlying an exporting process endpoint to close.
 * No effect on open file endpoints. The file or socket may be reopened on a
 * subsequent message emission from the associated buffer.
 * 
 * @param exporter  an exporting process endpoint.
 */
void                fbExporterClose(
    fbExporter_t       *exporter);

/**
 * Allocate a collecting process endpoint for a named file. The underlying
 * file will be opened immediately.
 *
 * @param ctx       application context; for application use, retrievable
 *                  by fbCollectorGetContext
 * @param path      path of file to read, or "-" to read standard input.
 * @param err       An error description, set on failure.
 * @return a collecting process endpoint, or NULL on failure.
 */

fbCollector_t       *fbCollectorAllocFile(
    void                *ctx,
    const char          *path,
    GError              **err);

/**
 * Allocate a collecting process endpoint for an open file.
 *
 * @param ctx       application context; for application use, retrievable
 *                  by fbCollectorGetContext
 * @param fp      file pointer to file to read.
 * @return a collecting process endpoint.
 */

fbCollector_t       *fbCollectorAllocFP(
    void                *ctx,
    FILE                *fp);

/**
 * Retrieve the application context associated with a collector. This context
 * is taken from the ctx argument of fbCollectorAllocFile() or
 * fbCollectorAllocFP(), or passed out via the ctx argument to the 
 * appinit function argument to fbListenerAlloc().
 *
 * @param collector a collecting process endpoint.
 * @return the application context
 */

void                *fbCollectorGetContext(
    fbCollector_t       *collector);

/**
 * Close the file or socket underlying a collecting process endpoint.
 * No effect on open file endpoints. If the collector is attached to a 
 * buffer managed by a listener, the buffer will be removed from the
 * listener (that is, it will not be returned by subsequent fbListenerWait()
 * calls).
 * 
 * @param collector  a collecting process endpoint.
 */

void                fbCollectorClose(
    fbCollector_t       *collector);

/**
 * Allocate a listener. The listener will listen on a specified local endpoint,
 * and create a new collecting process endpoint and collection buffer for each
 * incoming connection. Each new buffer will be associated with a clone of
 * a given session state container. 
 *
 * The application may associate context with each created collecting process
 * endpoint, or veto a connection attempt, via a function colled on each
 * connection attempt passed in via the appinit parameter. If this function
 * will create application context, provide a function via the appfree parameter
 * which will free it.
 *
 * @param spec      local endpoint connection specifier
 * @param session   session state container to clone for each collection buffer
 *                  created by the listener
 * @param appinit   application connection initiation function. Called on each
 *                  collection attempt; vetoes connection attempts and creates
 *                  application context.
 * @param appfree   application context free function.
 * @param err       An error description, set on failure.
 * @return a new listener, or NULL on failure.
 */
 
fbListener_t        *fbListenerAlloc(
    fbConnSpec_t            *spec,
    fbSession_t             *session,
    fbListenerAppInit_fn    appinit,
    fbListenerAppFree_fn    appfree,
    GError                  **err);

/**
 * Free a listener. Stops listening on the local endpoint, and frees any
 * open buffers still managed by the listener.
 *
 * @param listener a listener
 */
 
void                fbListenerFree(
    fbListener_t            *listener);

/**
 * Wait on a listener. Accepts pending connections from exporting processes.
 * Returns the next collection buffer with available data to read; if the
 * collection buffer returned by the last call to fbListenerWait() is available,
 * it is preferred. Blocks forever (or until fbListenerInterrupt() is called) 
 * if no messages or connections are available.
 *
 * To effectively use fbListenerWait(), the application should set up an
 * session state container with internal templates, call fbListenerWait()
 * to accept a first connection, then read records from the collector buffer 
 * to end of message (FB_ERROR_EOM). At end of message, the application should 
 * then call fbListenerWait() to accept pending connections or switch to 
 * another collector buffer with available data. Note that each collector
 * buffer returned created by fbListenerWait() is set to manual mode using 
 * fBufSetAutomaticMode().
 *
 * @param listener  a listener
 * @param err       An error description, set on failure.
 * @return a collection buffer with available data, or NULL on failure.
 */

fBuf_t              *fbListenerWait(
    fbListener_t            *listener,
    GError                  **err);

/**
 * Cause the current or next call to fbListenerWait to unblock and return.
 * Use this from a thread or a signal handler to interrupt a blocked listener.
 *
 * @param listener listener to interrupt.
 */

void                fbListenerInterrupt(
    fbListener_t            *listener);

#endif
